@@ -1,17 +1,16 @@
-from grizzly.expression import ColRef
-from grizzly.sqlgenerator import SQLGenerator
 from grizzly.generator import GrizzlyGenerator
+from grizzly.sqlgenerator import SQLGenerator
 
 import logging
-
+from typing import List
 logger = logging.getLogger(__name__)
 
 class RelationalExecutor(object):
   
   def __init__(self, connection, queryGenerator=SQLGenerator()):
-    super().__init__()
     self.connection = connection
     self.queryGenerator = queryGenerator
+    super().__init__()
 
   def generate(self, df):
     return self.queryGenerator.generate(df)
@@ -37,13 +36,17 @@ class RelationalExecutor(object):
   def close(self):
     self.connection.close()
 
+  def fetchone(self, df):
+    rs = self.execute(df)
+    return rs.fetchone()
+
   def collect(self, df, includeHeader):
     rs = self.execute(df)
 
     tuples = []
 
     if includeHeader:
-      cols = [dec[0] for dec in rs.description]
+      cols = RelationalExecutor.__getHeader(rs)
       tuples.append(cols)
 
     for row in rs:
@@ -51,12 +54,42 @@ class RelationalExecutor(object):
 
     return tuples
 
+  def iterator(self, df, includeHeader):
+    '''
+    Returns an iterator over the result of the DF
+    If includeHeader is true, the first row to be returned are the column names
+    '''
+    rs = self.execute(df)
 
-  def table(self,df):
+    if includeHeader:
+      yield RelationalExecutor.__getHeader(rs)
+
+    for row in rs:
+      yield row
+
+  @staticmethod
+  def __getHeader(rs) -> List[str]:
+    if rs.description:
+      cols = [dec[0] for dec in rs.description]
+    else:
+      cols = []
+    return cols
+
+  def table(self,df,limit=10):
     rs = self.execute(df)
     import beautifultable
     table = beautifultable.BeautifulTable()
+
+    header = RelationalExecutor.__getHeader(rs)
+    table.columns.header = header
+
+    cnt = 0
     for row in rs:
+
+      if cnt > limit:
+        break
+
+      cnt += 1
       table.rows.append(row)
 
     rs.close()
@@ -65,10 +98,7 @@ class RelationalExecutor(object):
   def toString(self, df, delim=",", pretty=False, maxColWidth=20, limit=20):
     rs = self.execute(df)
 
-    if rs.description:
-      cols = [dec[0] for dec in rs.description]
-    else:
-      cols = []
+    cols = RelationalExecutor.__getHeader(rs)
 
     if not pretty:
       strings = [delim.join(cols)]
@@ -135,30 +165,18 @@ class RelationalExecutor(object):
     # print(sql)
     return self._execute(sql)
 
-  def _execAgg(self, df, col, func):
-    """
-    Actually compute the aggregation function.
-
-    If we have a GROUP BY operation, the aggregation is only stored
-    as a transformation and needs to be executed using show() or similar.
-
-    If no grouping exists, we want to compute the aggregate over the complete
-    table and return the scalar result directly
-    """
-
-    return self._doExecAgg(df, col, func)
-
-  def _gen_agg(self, df, col, func):
-    return self.queryGenerator._generateAggCode(df, col, func)
-
-  def _doExecAgg(self, df, col, func):
+  def _execAgg(self, df, f):
     """
     Really executes the aggregation and returns the single result
     """
-    (pre, aggQry) = self.queryGenerator._generateAggCode(df, col, func)
+    (pre, aggQry) = self.queryGenerator._generateAggCode(df, f)
     for pq in pre:
       self._execute(pq).close()
     # execute an SQL query and get the result set
     rs = self._execute(aggQry)
     #fetch first (and only) row, return first column only
-    return rs.fetchone()[0]  
\ No newline at end of file
+    return rs.fetchone()[0]  
+
+  def _gen_agg(self, df, func):
+    return self.queryGenerator._generateAggCode(df, func)
+
